@@ -1,5 +1,5 @@
 // ----------------------------------------------------------------------------|
-//     Copyright (C) 2012-2016 Mohammadi A. and Wit C. E.
+//     Copyright (C) 2012-2017 A. (Reza) Mohammadi
 //
 //     This file is part of BDgraph package.
 //
@@ -8,7 +8,7 @@
 //     Software Foundation; see <https://cran.r-project.org/web/licenses/GPL-3>.
 //
 //     Maintainer:
-//     Abdolreza Mohammadi: a.mohammadi@rug.nl or a.mohammadi@uvt.nl
+//     Reza Mohammadi: a.mohammadi@rug.nl or a.mohammadi@uvt.nl
 // ----------------------------------------------------------------------------|
 #include <sstream>
 #include <string>        // std::string, std::to_string
@@ -30,32 +30,19 @@ extern "C" {
 // for case D = I_p 
 // it is for Bayesian model averaging (MA)
 // ----------------------------------------------------------------------------|
-void ggm_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
+void ggm_bdmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double Ts[], double K[], int *p, 
 			 double K_hat[], double p_links[],
 			 int *b, int *b_star, double Ds[], int *print )
 {
 	int print_c = *print, iteration = *iter, burn_in = *burnin;
 	int index_selected_edge, selected_edge_i, selected_edge_j, selected_edge_ij;
-	int i, j, ij, one = 1, dim = *p, pxp = dim * dim, p1 = dim - 1, p2x2 = ( dim - 2 ) * 2;
-
+	int counter = 0, ip, i, j, ij, one = 1, dim = *p, pxp = dim * dim;
+	int qp = dim * ( dim - 1 ) / 2;
 	double Dsij, sum_weights = 0.0, weight_C, sum_rates;
 	
 	vector<double> p_links_Cpp( pxp, 0.0 ); 
 	vector<double> K_hat_Cpp( pxp, 0.0 ); 
 
-	//-- allocation for birth/death rates 
-	vector<double> K121( 4 ); 
-	vector<double> Kj12( p1 );              // K[j, -j]
-	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
-	vector<double> sigmaj22( p1 * p1 );     // sigma[-j, -j]
-	vector<double> Kj12xK22_inv( p1 ); 
-
-	vector<double> K21( p2x2 );                          // K[-e, e]
-	vector<double> sigma21( p2x2 );                      // sigma[-e, e]
-	vector<double> sigma22( ( dim - 2 ) * ( dim - 2 ) ); // sigma[-e, -e]
-	vector<double> sigma11_inv( 4 );                     // inv( sigma[e, e] )
-	vector<double> sigma21xsigma11_inv( p2x2 ); 
-	vector<double> K12xK22_inv( p2x2 );   
 	// -- allocation for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
@@ -72,7 +59,6 @@ void ggm_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double K[], in
 	inverse( &copyK[0], &sigma[0], &dim );			
 
 	// Counting size of notes
-	int ip;
 	vector<int> size_node( dim, 0 );
 	for( i = 0; i < dim; i++ )
 	{
@@ -81,24 +67,26 @@ void ggm_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double K[], in
 	}
 	
 	// For finding the index of rates 
-	int qp = dim * ( dim - 1 ) / 2;
 	vector<double> rates( qp );
-	vector<int> index_rates_row( qp );
-	vector<int> index_rates_col( qp );
-	int counter = 0;
+	vector<int> index_row( qp );
+	vector<int> index_col( qp );
 	vector<double> Dsijj( pxp ); 
 	for( j = 1; j < dim; j++ )
 		for( i = 0; i < j; i++ )
 		{
-			index_rates_row[counter] = i;
-			index_rates_col[counter] = j;
-			counter++;
+			if( g_space[ j * dim + i ] )
+			{
+				index_row[counter] = i;
+				index_col[counter] = j;
+				counter++;
+			}
 			
 			// for calculating the birth/death rates
 			ij        = j * dim + i;
 			Dsij      = Ds[ij];
 			Dsijj[ij] = Dsij * Dsij / Ds[j * dim + j]; 
 		}
+	int sub_qp = counter;
 
 //-- Main loop for birth-death MCMC -------------------------------------------| 
 	GetRNGstate();
@@ -108,16 +96,12 @@ void ggm_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double K[], in
 		
 //----- STEP 1: calculating birth and death rates -----------------------------|		
 
-		rates_bdmcmc( &rates[0], G, Ds, &Dsijj[0],
-                      &sigma[0], &sigma21[0], &sigma22[0], &sigmaj12[0], &sigmaj22[0],    
-                      &K[0], &K21[0], &K121[0], &Kj12[0], 
-                      &K12xK22_inv[0], &Kj12xK22_inv[0], &sigma11_inv[0], &sigma21xsigma11_inv[0],  
-                      b, &dim );
+		rates_bdmcmc_parallel( &rates[0], G, &index_row[0], &index_col[0], &sub_qp, Ds, &Dsijj[0], &sigma[0], &K[0], b, &dim );
 
 		// Selecting an edge based on birth and death rates
-		select_edge( &rates[0], &index_selected_edge, &sum_rates, &qp );
-		selected_edge_i = index_rates_row[ index_selected_edge ];
-		selected_edge_j = index_rates_col[ index_selected_edge ];
+		select_edge( &rates[0], &index_selected_edge, &sum_rates, &sub_qp );
+		selected_edge_i = index_row[ index_selected_edge ];
+		selected_edge_j = index_col[ index_selected_edge ];
 
 //----- saving result ---------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -168,14 +152,14 @@ void ggm_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double K[], in
 // for case D = I_p 
 // it is for maximum a posterior probability estimation (MAP)
 // ----------------------------------------------------------------------------|
-void ggm_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
+void ggm_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double Ts[], double K[], int *p, 
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
 			 int *b, int *b_star, double Ds[], int *print )
 {
 	int print_c = *print, iteration = *iter, burn_in = *burnin, count_all_g = 0;
 	int index_selected_edge, selected_edge_i, selected_edge_j, selected_edge_ij, size_sample_graph = *size_sample_g;
-	int i, j, ij, one = 1, dim = *p, pxp = dim * dim, p1 = dim - 1, p2x2 = ( dim - 2 ) * 2;
+	int i, j, ij, one = 1, dim = *p, pxp = dim * dim;
 
 	double Dsij, sum_weights = 0.0, weight_C, sum_rates;
 	
@@ -184,19 +168,6 @@ void ggm_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double K[], i
 	string string_g;
 	vector<string> sample_graphs_C( iteration - burn_in );
 
-	//-- allocation for birth/death rates 
-	vector<double> K121( 4 ); 
-	vector<double> Kj12( p1 );              // K[j, -j]
-	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
-	vector<double> sigmaj22( p1 * p1 );     // sigma[-j, -j]
-	vector<double> Kj12xK22_inv( p1 ); 
-
-	vector<double> K21( p2x2 );                          // K[-e, e]
-	vector<double> sigma21( p2x2 );                      // sigma[-e, e]
-	vector<double> sigma22( ( dim - 2 ) * ( dim - 2 ) ); // sigma[-e, -e]
-	vector<double> sigma11_inv( 4 );                     // inv( sigma[e, e] )
-	vector<double> sigma21xsigma11_inv( p2x2 ); 
-	vector<double> K12xK22_inv( p2x2 );   
 	// -- allocation for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
@@ -226,22 +197,26 @@ void ggm_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double K[], i
 	
 	// For finding the index of rates 
 	vector<double> rates( qp );
-	vector<int> index_rates_row( qp );
-	vector<int> index_rates_col( qp );
+	vector<int> index_row( qp );
+	vector<int> index_col( qp );
 	int counter = 0;
 	vector<double> Dsijj( pxp ); 
 	for( j = 1; j < dim; j++ )
 		for( i = 0; i < j; i++ )
 		{
-			index_rates_row[counter] = i;
-			index_rates_col[counter] = j;
-			counter++;
+			if( g_space[ j * dim + i ] )
+			{
+				index_row[counter] = i;
+				index_col[counter] = j;
+				counter++;
+			}
 			
 			// for calculating the birth/death rates
 			ij        = j * dim + i;
 			Dsij      = Ds[ij];
 			Dsijj[ij] = Dsij * Dsij / Ds[j * dim + j]; 
 		}
+	int sub_qp = counter;
 
 //-- Main loop for birth-death MCMC -------------------------------------------| 
 	GetRNGstate();
@@ -251,16 +226,12 @@ void ggm_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double K[], i
 		
 //----- STEP 1: calculating birth and death rates -----------------------------|		
 
-		rates_bdmcmc( &rates[0], G, Ds, &Dsijj[0],
-                      &sigma[0], &sigma21[0], &sigma22[0], &sigmaj12[0], &sigmaj22[0],    
-                      &K[0], &K21[0], &K121[0], &Kj12[0], 
-                      &K12xK22_inv[0], &Kj12xK22_inv[0], &sigma11_inv[0], &sigma21xsigma11_inv[0],  
-                      b, &dim );
+		rates_bdmcmc_parallel( &rates[0], G, &index_row[0], &index_col[0], &sub_qp, Ds, &Dsijj[0], &sigma[0], &K[0], b, &dim );
 		
 		// Selecting an edge based on birth and death rates
-		select_edge( &rates[0], &index_selected_edge, &sum_rates, &qp );
-		selected_edge_i = index_rates_row[ index_selected_edge ];
-		selected_edge_j = index_rates_col[ index_selected_edge ];
+		select_edge( &rates[0], &index_selected_edge, &sum_rates, &sub_qp );
+		selected_edge_i = index_row[ index_selected_edge ];
+		selected_edge_j = index_col[ index_selected_edge ];
 
 //----- saving result ---------------------------------------------------------|	
 		counter = 0;	
@@ -340,32 +311,19 @@ void ggm_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double K[], i
 // for D = I_p 
 // it is for Bayesian model averaging
 // ----------------------------------------------------------------------------|
-void ggm_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
+void ggm_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], int g_space[], double Ts[], double K[], int *p, 
 			 double K_hat[], double p_links[],
 			 int *b, int *b_star, double Ds[], int *multi_update, int *print )
 {
 	int print_c = *print, iteration = *iter, burn_in = *burnin, multi_update_C = *multi_update;
 	int selected_edge_i, selected_edge_j, selected_edge_ij;
-	int i, j,ij, one = 1, dim = *p, pxp = dim * dim, p1 = dim - 1, p2x2 = ( dim - 2 ) * 2;
+	int i, j,ij, one = 1, dim = *p, pxp = dim * dim;
 
 	double Dsij, sum_weights = 0.0, weight_C, sum_rates;
 
 	vector<double> p_links_Cpp( pxp, 0.0 ); 
 	vector<double> K_hat_Cpp( pxp, 0.0 ); 
 
-	//-- allocation for birth/death rates 
-	vector<double> K121( 4 ); 
-	vector<double> Kj12( p1 );              // K[j, -j]
-	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
-	vector<double> sigmaj22( p1 * p1 );     // sigma[-j, -j]
-	vector<double> Kj12xK22_inv( p1 ); 
-
-	vector<double> K21( p2x2 );                          // K[-e, e]
-	vector<double> sigma21( p2x2 );                      // sigma[-e, e]
-	vector<double> sigma22( ( dim - 2 ) * ( dim - 2 ) ); // sigma[-e, -e]
-	vector<double> sigma11_inv( 4 );                     // inv( sigma[e, e] )
-	vector<double> sigma21xsigma11_inv( p2x2 ); 
-	vector<double> K12xK22_inv( p2x2 );   
 	// -- allocation for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
@@ -393,22 +351,26 @@ void ggm_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[], d
 	// For finding the index of rates 
 	int qp = dim * ( dim - 1 ) / 2;
 	vector<double> rates( qp );
-	vector<int> index_rates_row( qp );
-	vector<int> index_rates_col( qp );
+	vector<int> index_row( qp );
+	vector<int> index_col( qp );
 	int counter = 0;
 	vector<double> Dsijj( pxp ); 
 	for( j = 1; j < dim; j++ )
 		for( i = 0; i < j; i++ )
 		{
-			index_rates_row[counter] = i;
-			index_rates_col[counter] = j;
-			counter++;
+			if( g_space[ j * dim + i ] )
+			{
+				index_row[counter] = i;
+				index_col[counter] = j;
+				counter++;
+			}
 			
 			// for calculating the birth/death rates
 			ij        = j * dim + i;
 			Dsij      = Ds[ij];
 			Dsijj[ij] = Dsij * Dsij / Ds[j * dim + j]; 
 		}
+	int sub_qp = counter;
 
 	int size_index = multi_update_C;
 	vector<int> index_selected_edges( multi_update_C );
@@ -421,14 +383,10 @@ void ggm_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[], d
 		
 //----- STEP 1: calculating birth and death rates -----------------------------|		
 
-		rates_bdmcmc( &rates[0], G, Ds, &Dsijj[0],
-                      &sigma[0], &sigma21[0], &sigma22[0], &sigmaj12[0], &sigmaj22[0],    
-                      &K[0], &K21[0], &K121[0], &Kj12[0], 
-                      &K12xK22_inv[0], &Kj12xK22_inv[0], &sigma11_inv[0], &sigma21xsigma11_inv[0],  
-                      b, &dim );
+		rates_bdmcmc_parallel( &rates[0], G, &index_row[0], &index_col[0], &sub_qp, Ds, &Dsijj[0], &sigma[0], &K[0], b, &dim );
 
 		// Selecting multiple edges based on birth and death rates
-		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &qp );
+		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &sub_qp );
 
 //----- Saving result ---------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -448,8 +406,8 @@ void ggm_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[], d
 		// Updating graph based on selected edges
 		for ( i = 0; i < size_index; i++ )
 		{
-			selected_edge_i = index_rates_row[ index_selected_edges[i] ];
-			selected_edge_j = index_rates_col[ index_selected_edges[i] ];
+			selected_edge_i = index_row[ index_selected_edges[i] ];
+			selected_edge_j = index_col[ index_selected_edges[i] ];
 			
 			selected_edge_ij    = selected_edge_j * dim + selected_edge_i;
 			G[selected_edge_ij] = 1 - G[selected_edge_ij];
@@ -485,14 +443,14 @@ void ggm_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[], d
 // for D = I_p 
 // it is for maximum a posterior probability estimation (MAP)
 // ----------------------------------------------------------------------------|
-void ggm_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts[], double K[], int *p, 
+void ggm_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_space[], double Ts[], double K[], int *p, 
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g, int *counter_all_g,
 			 int *b, int *b_star, double Ds[], int *multi_update, int *print )
 {
 	int print_c = *print, multi_update_C = *multi_update, iteration = *iter, burn_in = *burnin;
 	int count_all_g = *counter_all_g, selected_edge_i, selected_edge_j, selected_edge_ij, size_sample_graph = *size_sample_g;
-	int i, j,ij, one = 1, dim = *p, pxp = dim * dim, p1 = dim - 1, p2x2 = ( dim - 2 ) * 2;
+	int i, j,ij, one = 1, dim = *p, pxp = dim * dim;
 	
 	double Dsij, sum_weights = 0.0, weight_C, sum_rates;
 	bool this_one;
@@ -500,19 +458,6 @@ void ggm_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts[],
 	string string_g;
 	vector<string> sample_graphs_C( iteration - burn_in );
 
-	//-- allocation for birth/death rates 
-	vector<double> K121( 4 ); 
-	vector<double> Kj12( p1 );              // K[j, -j]
-	vector<double> sigmaj12( p1 );          // sigma[-j, j]  
-	vector<double> sigmaj22( p1 * p1 );     // sigma[-j, -j]
-	vector<double> Kj12xK22_inv( p1 ); 
-
-	vector<double> K21( p2x2 );                          // K[-e, e]
-	vector<double> sigma21( p2x2 );                      // sigma[-e, e]
-	vector<double> sigma22( ( dim - 2 ) * ( dim - 2 ) ); // sigma[-e, -e]
-	vector<double> sigma11_inv( 4 );                     // inv( sigma[e, e] )
-	vector<double> sigma21xsigma11_inv( p2x2 ); 
-	vector<double> K12xK22_inv( p2x2 );   
 	// -- allocation for rgwish_sigma 
 	vector<double> sigma_start( pxp ); 
 	vector<double> inv_C( pxp ); 
@@ -542,22 +487,26 @@ void ggm_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts[],
 	
 	// For finding the index of rates 
 	vector<double> rates( qp );
-	vector<int> index_rates_row( qp );
-	vector<int> index_rates_col( qp );
+	vector<int> index_row( qp );
+	vector<int> index_col( qp );
 	int counter = 0;
 	vector<double> Dsijj( pxp ); 
 	for( j = 1; j < dim; j++ )
 		for( i = 0; i < j; i++ )
 		{
-			index_rates_row[counter] = i;
-			index_rates_col[counter] = j;
-			counter++;
+			if( g_space[ j * dim + i ] )
+			{
+				index_row[counter] = i;
+				index_col[counter] = j;
+				counter++;
+			}
 			
 			// for calculating the birth/death rates
 			ij        = j * dim + i;
 			Dsij      = Ds[ij];
 			Dsijj[ij] = Dsij * Dsij / Ds[j * dim + j]; 
 		}
+	int sub_qp = counter;
 
 	int size_index = multi_update_C;
 	vector<int> index_selected_edges( multi_update_C );
@@ -570,14 +519,10 @@ void ggm_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts[],
 		
 //----- STEP 1: calculating birth and death rates -----------------------------|		
 
-		rates_bdmcmc( &rates[0], G, Ds, &Dsijj[0],
-                      &sigma[0], &sigma21[0], &sigma22[0], &sigmaj12[0], &sigmaj22[0],    
-                      &K[0], &K21[0], &K121[0], &Kj12[0], 
-                      &K12xK22_inv[0], &Kj12xK22_inv[0], &sigma11_inv[0], &sigma21xsigma11_inv[0],  
-                      b, &dim );
+		rates_bdmcmc_parallel( &rates[0], G, &index_row[0], &index_col[0], &sub_qp, Ds, &Dsijj[0], &sigma[0], &K[0], b, &dim );
 				
 		// Selecting multiple edges based on birth and death rates
-		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &qp );
+		select_multi_edges( &rates[0], &index_selected_edges[0], &size_index, &sum_rates, &multi_update_C, &sub_qp );
 
 //----- Saving result ---------------------------------------------------------|	
 		counter = 0;	
@@ -621,8 +566,8 @@ void ggm_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts[],
 		// Updating graph based on selected edges
 		for ( i = 0; i < size_index; i++ )
 		{
-			selected_edge_i = index_rates_row[ index_selected_edges[i] ];
-			selected_edge_j = index_rates_col[ index_selected_edges[i] ];
+			selected_edge_i = index_row[ index_selected_edges[i] ];
+			selected_edge_j = index_col[ index_selected_edges[i] ];
 			
 			selected_edge_ij    = selected_edge_j * dim + selected_edge_i;
 			G[selected_edge_ij] = 1 - G[selected_edge_ij];
